// This file implements a simple Codon convenience class.
// 
// Codon Optimizer - a reference implementation of the CAI optimization 
// and motif engineering algorithm by Anne Condon and Chris Thachuk. 
//
// Copyright (C) 2012 Chris Thachuk
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _CODON_H_
#define _CODON_H_

#include <iostream>

#include "const.h"

namespace oligodesign {

class Codon {
public:
  Codon() {
    b1 = b2 = b3 = DNA_A;
    freq = 0.0;
    fit = 0.0;
  }
      
  Codon(const char *codon_str, double frequency, double fitness) {
    int bases[3];
    for (int i = 0; i < 3; i++) {
      switch (codon_str[i]) {
      case 'A':
        bases[i] = DNA_A;
        break;
      case 'T':
        bases[i] = DNA_T;
        break;
      case 'G':
        bases[i] = DNA_G;
        break;
      case 'C':
        bases[i] = DNA_C;
        break;
      }
    }
    freq = frequency;
    fit = fitness;
    b1 = bases[0];
    b2 = bases[1];
    b3 = bases[2];
  }

  Codon(int base1, int base2, int base3, double frequency, double fitness) {
    freq = frequency;
    fit = fitness;
    b1 = base1;
    b2 = base2;
    b3 = base3;
  }

  int operator<(const Codon &rhs) const {
    if (this->freq < rhs.freq) return 1;
    if (this->freq == rhs.freq &&
        this->b1 < rhs.b1) return 1;
    if (this->freq == rhs.freq &&
        this->b1 == rhs.b1 &&
        this->b2 < rhs.b2) return 1;
    if (this->freq == rhs.freq &&
        this->b1 == rhs.b1 &&
        this->b2 == rhs.b2 &&
        this->b3 < rhs.b3) return 1;
    return 0;
  }

  double freq;
  double fit;
  int b1;
  int b2;
  int b3;
};

} // end namespace oligodesign

#endif /* _CODON_H_ */
