// This file implements common utilities used throughout the namespace.
// 
// Codon Optimizer - a reference implementation of the CAI optimization 
// and motif engineering algorithm by Anne Condon and Chris Thachuk. 
//
// Copyright (C) 2012 Chris Thachuk
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#ifndef _UTIL_H_
#define _UTIL_H_

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <boost/algorithm/string.hpp>

using namespace std;

namespace oligodesign {

bool SequenceFromFasta(const char *file_name, std::string &seqS, int index, std::string &seqName) {
  std::ifstream file(file_name, std::ios::in);
  std::ostringstream fastaSeq;
  std::string sequence;
  std::string buff;
  std::string nameBuff;

  int curIndex = 0;
  seqS = "";

  if (file.is_open()) {
    while (! file.eof()) {
      file >> buff;
      if (! file.eof()) {

        // '>' denotes the title of a new sequence
        if (buff[0] == '>') {
          curIndex++;
          nameBuff = buff;
        } 
        // treat '#' as comment lines
        else if (buff[0] == '#') {
          continue;
        } 
        // break out of reading file if we've passed the index
        else if (curIndex > index) {
          break;
        }
        // add line to sequence buffer if this is the correct index
        else if (curIndex == index) {
          seqName = nameBuff;
          boost::trim(buff);
          fastaSeq << buff;
        }
      }
    }
    seqS = fastaSeq.str();
    boost::to_upper(seqS);
    file.close();
  }
  else {
    std::cerr << "Unable to open file: " << file_name << std::endl;
  }

  return seqS != "";
}

} // end namespace oligodesign

#endif /* _UTIL_H_ */
